package net.nighttale.hibernate;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Properties;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.dialect.Dialect;
import net.sf.hibernate.id.IdentifierGenerationException;
import net.sf.hibernate.type.Type;
import net.sf.hibernate.util.PropertiesHelper;
import net.sf.hibernate.util.StringHelper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * An <tt>IdentifierGenerator</tt> that uses a database
 * table to store the last generated value.
 * <br>
 * The hi value MUST be fetched in a seperate transaction
 * to the <tt>Session</tt> transaction so the generator must 
 * be able to obtain a new connection and commit it. Hence 
 * this implementation may not be used when Hibernate is 
 * fetching connections from an application server datasource 
 * or when the user is supplying connections.<br>
 * <br>
 * Mapping parameters supported: table, column_name, column_value, counter_name
 * 
 * @see MuliplteTableHiLoGenerator
 * @author Dejan Bosanac
 */
public class MultipleTableGenerator extends TableGenerator {

	/**
	 * The table parameter
	 */
	public static final String TABLE = "table";	
	
	/**
	 * The column parameter
	 */
	public static final String COLUMN_NAME = "column_name";
	
	public static final String COLUMN_VALUE = "column_value";
	
	public static final String COUNTER_NAME = "counter_name";
	

	private static final Log log = LogFactory.getLog(TableGenerator.class);

	String columnValue;
	String counterName;

	public void configure(Type type, Properties params, Dialect dialect) {
	
		this.tableName = PropertiesHelper.getString(TABLE, params, "counters");
		this.columnName = PropertiesHelper.getString(COLUMN_NAME, params, "name");
		this.columnValue = PropertiesHelper.getString(COLUMN_VALUE, params, "value");
		this.counterName = PropertiesHelper.getString(COUNTER_NAME, params, "default");
		String schemaName = params.getProperty(SCHEMA);
		if ( schemaName!=null && tableName.indexOf(StringHelper.DOT)<0 ) 
			tableName = schemaName + '.' + tableName;
	
		query = "select " + columnValue + " from " + tableName + " where " + columnName + " = '" + counterName + "'";
		if ( dialect.supportsForUpdate() ) query += " for update";
		update = "update " + tableName + " set " + columnValue + " = ? where " + columnName + " = '" + counterName + "'";
		
		returnClass = type.getReturnedClass();
	}
	
	public PreparedStatement prepareUpdate(Connection conn, int result) 
		throws SQLException {
		PreparedStatement updateStatement = conn.prepareStatement(update);
		updateStatement.setInt( 1, result + 1 );
		
		return updateStatement;		
	}

	public String[] sqlCreateStrings(Dialect dialect) throws HibernateException {
		return new String[] {
			"create table " + tableName 
			+ " ( "
			+ " " + columnName + " " + dialect.getTypeName(Types.VARCHAR, 255) + " NOT NULL " + ", "
			+ " " + columnValue + " " + dialect.getTypeName(Types.INTEGER) + ", "
			+ " primary key(" + columnName + ")"
			+ ") "
			,
			"insert into " + tableName + " values ('" + counterName + "', 0 )"
		};
	}

}

	

